@@ -1,8 +1,18 @@
 # coding=utf-8
+from __future__ import annotations
 
 import json
-from copy import copy
 from collections import defaultdict
+from copy import copy
+from enum import Enum
+from inspect import isfunction
+from typing import TYPE_CHECKING, Optional, Callable
+
+from olo.orm_types import TrackedValue
+from olo.types.json import JSONLike
+
+if TYPE_CHECKING:
+    from olo.model import Model
 
 from olo.compat import int_types, iteritems, izip, str_types
 from olo.interfaces import SQLASTInterface
@@ -13,8 +23,6 @@ from olo.utils import transform_type, missing, type_checker
 from olo.funcs import attach_func_method
 from olo.errors import ValidationError, DbFieldVersionError
 from olo.migration import MigrationVersion
-from olo.context import context
-
 
 __all__ = ['Field', 'UnionField', 'DbField']
 
@@ -23,7 +31,7 @@ class BaseField(object):
 
     id = 0
 
-    def __init__(self, type,
+    def __init__(self, type_,
                  default=None, name=None,
                  parser=None, deparser=None,
                  primary_key=False, on_update=None,
@@ -41,7 +49,7 @@ class BaseField(object):
         self.id = self.__class__.id + 1
         self.__class__.id = self.id
 
-        self.type = type
+        self.type = type_
         self.name = name
         self.default = default
         self._parser = (
@@ -58,7 +66,7 @@ class BaseField(object):
         self.encrypt = encrypt
         self.input = input
         self.output = output
-        self.noneable = noneable
+        self.noneable = noneable if not primary_key else False
         self.attr_name = attr_name or name
         self.version = version
         self.length = length
@@ -66,11 +74,17 @@ class BaseField(object):
         self.charset = charset
         self._alias_name = None
         self._model_ref = lambda: None
+        self.AES_KEY = ''
+        self._getter = None
+        self._setter = None
+
+        if self.choices is None and isinstance(self.type, type) and issubclass(self.type, Enum):
+            self.choices = self.type
 
     def __hash__(self):
         return self.id
 
-    def get_model(self):
+    def get_model(self) -> Optional[Model]:
         if not self._model_ref:
             return
         return self._model_ref()
@@ -103,7 +117,7 @@ class BaseField(object):
         except (TypeError, ValueError):
             return value
 
-    def is_primary_key(self):
+    def is_primary_key(self) -> bool:
         return self._primary_key
 
     def validate(self, value):
@@ -141,9 +155,26 @@ class BaseField(object):
     def _get_parsed_data(cls, obj):
         return obj._parsed_data
 
+    def mark_dirty(self, obj):
+        obj._dirty_fields.add(self.attr_name)
+
+    def getter(self, func):
+        self._getter = func
+        return self
+
+    def setter(self, func):
+        self._setter = func
+        return self
+
     def __get__(self, obj, objtype):
         if obj is None:
             return self
+        v = self._get(obj, objtype)
+        if self._getter is not None:
+            return self._getter(obj, v)
+        return v
+
+    def _get(self, obj, objtype):
         attr_name = self.attr_name
         data = self._get_data(obj)
         parsed_data = self._get_parsed_data(obj)
@@ -174,6 +205,11 @@ class BaseField(object):
                     self.name
                 )
             )
+        if self._setter is not None:
+            value = self._setter(obj, value)
+        return self._set(obj, value)
+
+    def _set(self, obj, value):
         if obj._orig is None:
             obj._set_orig()
         attr_name = self.attr_name
@@ -189,7 +225,7 @@ class BaseField(object):
             attr_name not in obj._dirty_fields and
             not obj._olo_is_new
         ):
-            obj._dirty_fields.add(attr_name)
+            self.mark_dirty(obj)
         data[attr_name] = parsed_value
         parsed_data.pop(attr_name, None)
 
@@ -201,6 +237,13 @@ class BaseField(object):
         )
 
 
+def process_enum_seq(v):
+    if isinstance(v, (list, tuple, set)):
+        t = type(v)
+        return t(i.name if isinstance(i, Enum) else i for i in v)
+    return v
+
+
 @attach_func_method
 class Field(BaseField, UnaryOperationMixin, BinaryOperationMixin,
             SQLASTInterface):
@@ -211,6 +254,14 @@ class Field(BaseField, UnaryOperationMixin, BinaryOperationMixin,
     def __init__(self, *args, **kwargs):
         super(Field, self).__init__(*args, **kwargs)
 
+    def in_(self, other):
+        seq = process_enum_seq(other)
+        return super().in_(seq)
+
+    def not_in_(self, other):
+        seq = process_enum_seq(other)
+        return super().not_in_(seq)
+
     def alias(self, name):
         inst = self.clone()
         inst._alias_name = name
@@ -219,8 +270,6 @@ class Field(BaseField, UnaryOperationMixin, BinaryOperationMixin,
     def get_sql_ast(self):
         model = self.get_model()
         table_name = None if not model else model._get_table_name()
-        alias_mapping = context.table_alias_mapping or {}
-        table_name = alias_mapping.get(table_name, table_name)
         sql_ast = [
             'COLUMN',
             table_name,
@@ -241,22 +290,117 @@ class ConstField(Field):
         return ['VALUE', self._value]
 
 
-class UnionField(BaseField, SQLASTInterface):
+class UnionField(BaseField, UnaryOperationMixin, BinaryOperationMixin, SQLASTInterface):
+
+    UnaryExpression = UnaryExpression
+    BinaryExpression = BinaryExpression
 
     def __init__(self, *fields):
         self.fields = fields
         super(UnionField, self).__init__(Field)
-
-    def in_(self, other):
-        return BinaryExpression(self, other, 'IN')
-
-    def not_in_(self, other):
-        return BinaryExpression(self, other, 'NOT IN')
+        self.attr_name = '({})'.format(tuple(f.attr_name for f in self.fields))
 
     def get_sql_ast(self):
         return ['BRACKET'] + [f.get_sql_ast() for f in self.fields]
 
 
+def _get_json_type(i):
+    if isinstance(i, bool):
+        return 'boolean'
+    if isinstance(i, int):
+        return 'int'
+    if isinstance(i, float):
+        return 'double precision'
+    return 'text'
+
+
+class JSONField(Field):
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(JSONLike, *args, **kwargs)
+        self._args = args
+        self._kwargs = kwargs
+        self._path = []
+        self._type = None
+
+    def _clone(self):
+        inst = self.__class__(*self._args, **self._kwargs)
+        inst._path = list(self._path)
+        inst._type = self._type
+        return inst
+
+    def __getitem__(self, item):
+        inst = self.clone()
+        inst._path.append(item)
+        return inst
+
+    def __eq__(self, other):
+        inst = self.clone()
+        inst._type = _get_json_type(other)
+        return BinaryExpression(inst, other, '=')
+
+    def __ne__(self, other):
+        inst = self.clone()
+        inst._type = _get_json_type(other)
+        operator = '!='
+        if other is None:
+            operator = 'IS NOT'
+        return self.BinaryExpression(inst, other, operator)
+
+    def __gt__(self, other):
+        inst = self.clone()
+        inst._type = _get_json_type(other)
+        return self.BinaryExpression(inst, other, '>')
+
+    def __ge__(self, other):
+        inst = self.clone()
+        inst._type = _get_json_type(other)
+        return self.BinaryExpression(inst, other, '>=')
+
+    def __lt__(self, other):
+        inst = self.clone()
+        inst._type = _get_json_type(other)
+        return self.BinaryExpression(inst, other, '<')
+
+    def __le__(self, other):
+        inst = self.clone()
+        inst._type = _get_json_type(other)
+        return self.BinaryExpression(inst, other, '<=')
+
+    def contains_(self, item):
+        return BinaryExpression(self, item, '?')
+
+    def not_contains_(self, item):
+        return UnaryExpression(BinaryExpression(self, item, '?'), 'NOT', suffix=False)
+
+    def __and__(self, other):
+        return BinaryExpression(self, other, '?&')
+
+    def __or__(self, other):
+        return BinaryExpression(self, other, '?|')
+
+    def get_sql_ast(self):
+        model = self.get_model()
+        table_name = None if not model else model._get_table_name()
+        sql_ast = [
+            'COLUMN',
+            table_name,
+            self.name,
+            self._path,
+            self._type,
+        ]
+        if self._alias_name:
+            sql_ast = ['ALIAS', sql_ast, self._alias_name]
+        return sql_ast
+
+    def __get__(self, obj, objtype):
+        v = super().__get__(obj, objtype)
+        if obj is not None:
+            v = TrackedValue.make(obj, self.attr_name, v)
+            obj._parsed_data[self.attr_name] = v
+        return v
+
+
 def _make_db_field_key(uuid, attr_name):
     return '{}/{}'.format(uuid, attr_name)
 
@@ -295,10 +439,11 @@ def _prefetch_db_data(obj):
     if not db_values:
         return
     for (entity, field, version), db_value in izip(pairs, db_values):
+        data = DbField._get_data(entity)
         if db_value is missing:
+            data[field.attr_name] = None
             continue
         value = _get_value_from_db_value(db_value, version, field)
-        data = DbField._get_data(entity)
         data[field.attr_name] = _process_value(value, field)
 
 
@@ -389,8 +534,7 @@ class DbField(BaseField):
             # beansdb 不支持 set None
             db.db_delete(db_key)
             return
-        else:
-            value = transform_type(value, self.type)
+        value = transform_type(value, self.type)
         db.db_set(db_key, value)
 
     def _delete_v0(self, obj):
@@ -405,14 +549,12 @@ class DbField(BaseField):
         db_key = self._get_db_field_key(obj)
         db.db_delete(db_key)
 
-    def __get__(self, obj, objtype):
-        if obj is None:
-            return self
+    def _get(self, obj, objtype):
         _prefetch_db_data(obj)
         attr_name = self.attr_name
         data = self._get_data(obj)
         if attr_name in data:
-            return super(DbField, self).__get__(obj, objtype)
+            return super(DbField, self)._get(obj, objtype)
         v = self.db_get(obj, objtype)
         data[attr_name] = v
         return v
@@ -452,7 +594,7 @@ class DbField(BaseField):
 
     def __delete__(self, obj):
         if obj is None:
-            raise AttributeError  # pragma: no cover
+            raise AttributeError(self.attr_name)  # pragma: no cover
         attr_name = self.attr_name
         if obj.before_update(**{
             attr_name: None
@@ -464,10 +606,83 @@ class DbField(BaseField):
         elif version == 1:
             self._delete_v1(obj)
         elif isinstance(version, MigrationVersion):
-            v = getattr(self, '_delete_v{}'.format(version.to_version))(obj)  # noqa
-            v = getattr(self, '_delete_v{}'.format(version.from_version))(obj)  # noqa
+            getattr(self, '_delete_v{}'.format(version.to_version))(obj)  # noqa
+            getattr(self, '_delete_v{}'.format(version.from_version))(obj)  # noqa
         data = self._get_data(obj)
         parsed_data = self._get_parsed_data(obj)
         data.pop(attr_name, None)
         parsed_data.pop(attr_name, None)
         obj.after_update()
+
+
+class BatchField(BaseField):
+    def __init__(self, type_, default=None, name=None):
+        if isfunction(type_):
+            type_ = object
+        super(BatchField, self).__init__(type_, default=default, name=name, noneable=True)
+
+    def getter(self, func_or_classmethod):
+        if isinstance(func_or_classmethod, classmethod):
+            func = func_or_classmethod.__func__
+        else:
+            func = func_or_classmethod
+        super(BatchField, self).getter(func)
+        return func_or_classmethod
+
+    def __get__(self, instance, owner):
+        if instance is None:
+            return self
+
+        data = self._get_data(instance)
+        v = data.get(self.name, missing)
+        if v is not missing:
+            return v
+
+        v = self._get(instance, owner)
+        data[self.name] = v
+        return v
+
+    def _get(self, instance, owner):
+        if self._getter is None:
+            raise AttributeError('batch field `{}.{}` has no getter!', owner.__name__, self.name)
+
+        session = instance._olo_qs
+        if session is None:
+            entities = [instance]  # pragma: no cover
+        else:
+            entities = session.entities
+
+        name = self.name
+        default = self.get_default()
+        res = self._getter(owner, entities)
+
+        entity_mapping = {
+            e._get_singleness_pk_value(): e
+            for e in entities
+        }
+
+        if isinstance(res, dict):
+
+            for pv, item in iteritems(entity_mapping):
+                if hasattr(item, '_olo_qs'):
+                    setattr(item, name, res.get(pv, default))
+
+            return res.get(instance._get_singleness_pk_value(), default)
+
+        if isinstance(res, list):
+
+            for idx, item in enumerate(entities):
+                if hasattr(item, '_olo_qs'):
+                    try:
+                        v = res[idx]
+                    except IndexError:
+                        v = default
+
+                    setattr(item, name, v)
+
+            try:
+                return res[instance._olo_qs_idx]
+            except IndexError:  # pragma: no cover
+                return default  # pragma: no cover
+
+        return default  # pragma: no cover
